package com.anguomob.flutter_plugin_anguo

import android.app.Activity
import android.content.Context
import android.text.TextUtils
import androidx.annotation.NonNull
import com.anguomob.flutter_plugin_anguo.utils.FlutterUtils
import com.anguomob.total.init.AnguoUtils

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import com.anguomob.total.utils.*


/** FlutterPluginAnguoPlugin */
class FlutterPluginAnguoPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel


    private lateinit var context: Context
    private lateinit var mActivity: Activity

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "anguo")
        channel.setMethodCallHandler(this)
        context = flutterPluginBinding.applicationContext
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {

        if (call.method != null) {
            // 根据方法名，分发不同的处理
            when (call.method) {
                "getUmChannel" -> {
                    result.success(UmUtils.getUmChannel())
                }
                "isAppNameOther" -> {
                    result.success(UmUtils.isAppNameOther(mActivity))
                }
                "getRealAppName" -> {
                    result.success(UmUtils.getRealAppName(mActivity))
                }
                "checkUpdate" -> {
                    result.success(AnguoUtils.checkUpDate(mActivity));
                }
                "fiveStar" -> {
                    result.success(SettingUtils.fiveStar(mActivity));
                }
                "openOrDownApk" -> {
                    openOrDownApk(call, result)

                }
                else -> {
                    result.notImplemented();
                }
            }
        } else {
            result.notImplemented();
        }
    }

    private fun openOrDownApk(
        call: MethodCall,
        result: Result
    ) {
        val packageName: String? = call.argument("packageName")
        val appDownUrl: String? = call.argument("appDownUrl")
        if (!TextUtils.isEmpty(packageName) && !TextUtils.isEmpty(appDownUrl)) {
            result.success(
                FlutterUtils.initOpenOrDownApk(
                    packageName!!,
                    appDownUrl,
                    mActivity
                )
            );
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    override fun onAttachedToActivity(p0: ActivityPluginBinding) {

    }

    override fun onDetachedFromActivityForConfigChanges() {

    }

    override fun onReattachedToActivityForConfigChanges(p0: ActivityPluginBinding) {
        mActivity = p0.activity;
    }

    override fun onDetachedFromActivity() {

    }
}
