package com.anguomob.flutter_plugin_anguo.utils

import android.Manifest
import android.app.Activity
import android.app.ActivityManager
import android.content.Context
import android.content.Intent
import android.content.pm.ApplicationInfo
import android.content.pm.PackageInfo
import android.content.pm.PackageManager
import android.graphics.drawable.Drawable
import android.net.Uri
import android.os.Build
import android.os.Environment
import android.os.StatFs
import android.text.TextUtils
import android.util.Log
import android.widget.Toast
import androidx.core.app.ActivityCompat
import androidx.core.content.PermissionChecker
import com.anguomob.flutter_plugin_anguo.R
import com.anguomob.total.dialog.HProgressDialogUtils
import com.anguomob.total.utils.Logger
import com.xuexiang.xupdate.XUpdate
import com.xuexiang.xupdate._XUpdate
import com.xuexiang.xupdate.service.OnFileDownloadListener
import java.io.*
import java.util.*
import java.util.regex.Pattern

object AppUtils {
    private const val TAG = "AppUtils"
    private val sdPermission: String = Manifest.permission.WRITE_EXTERNAL_STORAGE
    fun downByXUpdate(activity: Activity, down_app_url: String) {
        //请求本地存储权限，没有的话就返回
        if (ActivityCompat.checkSelfPermission(activity, sdPermission) == PermissionChecker.PERMISSION_DENIED) {
            ActivityCompat.requestPermissions(activity, arrayOf(sdPermission), 105)
            Toast.makeText(activity, "请授予存储权限后重试", Toast.LENGTH_LONG).show();
            return;
        }


        val path = Environment
                .getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS)
                .absolutePath;
        XUpdate.newBuild(activity)
                .apkCacheDir(path) //设置下载缓存的根目录
                .build()
                .download(down_app_url, object : OnFileDownloadListener {
                    //设置下载的地址和下载的监听
                    override fun onStart() {
                        Logger.e(TAG, "onStart: ")
                        HProgressDialogUtils.showHorizontalProgressDialog(activity, "下载进度", false)
                    }

                    override fun onProgress(progress: Float, total: Long) {
                        Logger.e(TAG, "onProgress:progress:$progress/total:$total ")
                        HProgressDialogUtils.setProgress(Math.round(progress * 100))
                    }

                    override fun onCompleted(file: File): Boolean {
                        Logger.e(TAG, "onCompleted: ")
                        HProgressDialogUtils.cancel()
                        //                                                    ToastUtils.toast("apk下载完毕，文件路径：" + file.getPath())
                        _XUpdate.startInstallApk(activity, file); //填写文件所在的路径
                        return false
                    }

                    override fun onError(throwable: Throwable) {
                        Logger.e(TAG, "onError: " + throwable.message.toString())
                        HProgressDialogUtils.cancel()
                    }
                })
    }



    /**
     * 获取包名
     *
     * @param context
     * @return
     */
    fun getPackageName(context: Context): String {
        return context.packageName
    }

    /**
     * 获取VersionName(版本名称)
     *
     * @param context
     * @return 失败时返回""
     */
    fun getVersionName(context: Context): String {
        val packageManager = getPackageManager(context)
        try {
            val packageInfo = packageManager.getPackageInfo(getPackageName(context), 0)
            return packageInfo.versionName
        } catch (e: PackageManager.NameNotFoundException) {
            e.printStackTrace()
        }
        return ""
    }

    /**
     * 获取VersionCode(版本号)
     *
     * @param context
     * @return 失败时返回-1
     */
    fun getVersionCode(context: Context): Int {
        val packageManager = getPackageManager(context)
        try {
            val packageInfo = packageManager.getPackageInfo(getPackageName(context), 0)
            return packageInfo.versionCode
        } catch (e: PackageManager.NameNotFoundException) {
            e.printStackTrace()
        }
        return -1
    }

    /**
     * 获取所有安装的应用程序,不包含系统应用
     *
     * @param context
     * @return
     */
    fun getInstalledPackages(context: Context): List<PackageInfo> {
        val packageManager = getPackageManager(context)
        val packageInfos = packageManager.getInstalledPackages(0)
        val packageInfoList: MutableList<PackageInfo> = ArrayList()
        for (i in packageInfos.indices) {
            if (packageInfos[i].applicationInfo.flags and ApplicationInfo.FLAG_SYSTEM == 0) {
                packageInfoList.add(packageInfos[i])
            }
        }
        return packageInfoList
    }

    /**
     * 获取应用程序的icon图标
     *
     * @param context
     * @return 当包名错误时，返回null
     */
    fun getApplicationIcon(context: Context): Drawable? {
        val packageManager = getPackageManager(context)
        try {
            val packageInfo = packageManager.getPackageInfo(getPackageName(context), 0)
            return packageInfo.applicationInfo.loadIcon(packageManager)
        } catch (e: PackageManager.NameNotFoundException) {
            e.printStackTrace()
        }
        return null
    }

    /**
     * 启动安装应用程序
     *
     * @param activity
     * @param path     应用程序路径
     */
    fun installApk(activity: Activity, path: String?) {
        val intent = Intent(Intent.ACTION_VIEW)
        intent.setDataAndType(
                Uri.fromFile(File(path)),
                "application/vnd.android.package-archive"
        )
        activity.startActivity(intent)
    }

    /**
     * 获取PackageManager对象
     *
     * @param context
     * @return
     */
    private fun getPackageManager(context: Context): PackageManager {
        return context.packageManager
    }// Default to return 1 core// Get directory containing CPU info

    // Filter to only list the devices we care about
    // Return the number of cores (virtual CPU devices)
// Check if filename is "cpu", followed by a single digit number// Private Class to display only CPU devices in the directory listing
    /*获取cpu 核心数*/
    val numCores: String
        get() {
            // Private Class to display only CPU devices in the directory listing
            class CpuFilter : FileFilter {
                override fun accept(pathname: File): Boolean {
                    // Check if filename is "cpu", followed by a single digit number
                    return if (Pattern.matches("cpu[0-9]", pathname.name)) {
                        true
                    } else false
                }
            }
            return try {
                // Get directory containing CPU info
                val dir = File("/sys/devices/system/cpu/")
                // Filter to only list the devices we care about
                val files = dir.listFiles(CpuFilter())
                // Return the number of cores (virtual CPU devices)
                files.size.toString() + ""
            } catch (e: Exception) {
                // Default to return 1 core
                "1"
            }
        }

    /**
     * 获取CPU型号
     *
     * @return
     */
    val cpuName: String?
        get() {
            val str1 = "/proc/cpuinfo"
            var str2 = ""
            try {
                val fr = FileReader(str1)
                val localBufferedReader = BufferedReader(fr)
                while (localBufferedReader.readLine().also { str2 = it } != null) {
                    if (str2.contains("Hardware")) {
                        return str2.split(":").toTypedArray()[1]
                    }
                }
                localBufferedReader.close()
            } catch (e: IOException) {
            }
            return null
        }

    /**
     * 获取CPU最大频率（单位KHZ）
     *
     * @return
     */
    // "/system/bin/cat" 命令行
    // "/sys/devices/system/cpu/cpu0/cpufreq/cpuinfo_max_freq" 存储最大频率的文件的路径
    //返回GHZ /10^6
    val maxCpuFreq: String
        get() {
            var result = ""
            val cmd: ProcessBuilder
            try {
                val args = arrayOf(
                        "/system/bin/cat",
                        "/sys/devices/system/cpu/cpu0/cpufreq/cpuinfo_max_freq"
                )
                cmd = ProcessBuilder(*args)
                val process = cmd.start()
                val `in` = process.inputStream
                val re = ByteArray(24)
                while (`in`.read(re) != -1) {
                    result = result + String(re)
                }
                `in`.close()
            } catch (ex: IOException) {
                ex.printStackTrace()
                result = "N/A"
            }
            if (result.length >= 6) {
                var res = 0.0
                return try {
                    res = getGroup(result).toFloat() / Math.pow(10.0, 6.0)
                    String.format("%.1f", res) + "GHZ"
                } catch (e: NumberFormatException) {
                    e.printStackTrace()
                    "N/A"
                }
            }
            return result.trim { it <= ' ' }
        }

    private fun getGroup(result: String): String {
        // 2188800
        //    ��������������������������������
        val compile = Pattern.compile(".+\\d+")
        val matcher = compile.matcher(result)
        val isFind = matcher.find()
        return if (isFind) {
            matcher.group(0)
        } else {
            "N/A"
        }
    }

    /**
     * 获得SD卡总大小
     *
     * @param context
     * @return
     */
    fun getSDTotalSize(context: Context?): Long {
        val path = Environment.getExternalStorageDirectory()
        val stat = StatFs(path.path)
        val blockSize = stat.blockSize.toLong()
        val totalBlocks = stat.blockCount.toLong()
        //        Formatter.formatFileSize(context,)
        return blockSize * totalBlocks
    }

    fun getTotalRam(context: Context?): Long { //GB
        val path = "/proc/meminfo"
        var firstLine: String? = null
        var totalRam: Long = 0
        try {
            val fileReader = FileReader(path)
            val br = BufferedReader(fileReader, 8192)
            firstLine = br.readLine().split("\\s+").toTypedArray()[1]
            br.close()
        } catch (e: Exception) {
            e.printStackTrace()
        }
        if (firstLine != null) {
            totalRam = Math.ceil(java.lang.Float.valueOf(firstLine).toDouble()).toLong()
        }
        return totalRam * 1024 //返回1GB/2GB/3GB/4GB
    }

    fun getScreenWidth(activity: Context): String {
//        return activity.getWindowManager().getDefaultDisplay().getWidth() + "";
        return activity.resources.displayMetrics.heightPixels.toString() + ""
    }

    fun getScreenHeight(activity: Context): String {
//        return activity.getWindowManager().getDefaultDisplay().getHeight() + "";
        return activity.resources.displayMetrics.widthPixels.toString() + ""
    }

    /**
     * 获取本地软件版本号名称
     */
    fun getLocalVersionName(ctx: Context): String {
        var localVersion = ""
        try {
            val packageInfo = ctx.applicationContext
                    .packageManager
                    .getPackageInfo(ctx.packageName, 0)
            localVersion = packageInfo.versionName
        } catch (e: PackageManager.NameNotFoundException) {
            e.printStackTrace()
        }
        return localVersion
    }
    //    ym 获取渠道号AnalyticsConfig.getChannel(this)
    /**
     * JAVA获得0-9,a-z,A-Z范围的随机数
     *
     * @param length 随机数长度
     * @return String
     */
    fun getRandomChar(length: Int): String {
        val chr = charArrayOf(
                '0',
                '1',
                '2',
                '3',
                '4',
                '5',
                '6',
                '7',
                '8',
                '9',
                'a',
                'b',
                'c',
                'd',
                'e',
                'f',
                'g',
                'h',
                'i',
                'j',
                'k',
                'l',
                'm',
                'n',
                'o',
                'p',
                'q',
                'r',
                's',
                't',
                'u',
                'v',
                'w',
                'x',
                'y',
                'z',
                'A',
                'B',
                'C',
                'D',
                'E',
                'F',
                'G',
                'H',
                'I',
                'J',
                'K',
                'L',
                'M',
                'N',
                'O',
                'P',
                'Q',
                'R',
                'S',
                'T',
                'U',
                'V',
                'W',
                'X',
                'Y',
                'Z',
                '~',
                '_',
                '!'
        )
        val random = Random()
        val buffer = StringBuffer()
        for (i in 0 until length) {
            buffer.append(chr[random.nextInt(62)])
        }
        return buffer.toString()
    }

    // 获取可用内存空间大小
    fun getAvailMemory(context: Context): Long {
        val am = context.getSystemService(Context.ACTIVITY_SERVICE) as ActivityManager
        val outInfo = ActivityManager.MemoryInfo()
        am.getMemoryInfo(outInfo)
        return Math.abs(outInfo.availMem)
    }

    // 获取总运存大小
    fun getTotalMemory(context: Context?): Long {
        val str1 = "/proc/meminfo" // 系统内存信息文件
        val str2: String
        val arrayOfString: Array<String>
        var initial_memory: Long = 0
        try {
            val localFileReader = FileReader(str1)
            val localBufferedReader = BufferedReader(localFileReader, 8192)
            str2 = localBufferedReader.readLine() // 读取meminfo第一行，系统总内存大小
            arrayOfString = str2.split("\\s+").toTypedArray()
            for (num in arrayOfString) {
                Log.i("tatolram:$str2", num + "\t")
            }
            initial_memory = java.lang.Long.valueOf(arrayOfString[1])
                    .toLong() * 1024 // 获得系统总内存，单位是KB，乘以1024转换为Byte
            localBufferedReader.close()
        } catch (e: Exception) {
        }
        return Math.abs(initial_memory)
    }

    fun startAPP(context: Context, appPackageName: String?) {
        try {
            val intent = context.packageManager.getLaunchIntentForPackage(
                    appPackageName!!
            )
            context.startActivity(intent)
        } catch (e: Exception) {
            //Toast.makeText(context, "没有安装", Toast.LENGTH_LONG).show();
        }
    }

    /**
     * SDCARD是否存
     */
    fun externalMemoryAvailable(): Boolean {
        return Environment.getExternalStorageState() ==
                Environment.MEDIA_MOUNTED
    }

    /**
     * 获取SDCARD剩余存储空间
     *
     * @return
     */
    val availableExternalMemorySize: Long
        get() = if (externalMemoryAvailable()) {
            val path = Environment.getExternalStorageDirectory()
            val stat = StatFs(path.path)
            var blockSize: Long = 0
            blockSize =
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR2) {
                        stat.blockSizeLong
                    } else {
                        stat.blockSize.toLong()
                    }
            var availableBlocks: Long = 0
            availableBlocks =
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR2) {
                        stat.availableBlocksLong
                    } else {
                        stat.availableBlocks.toLong()
                    }
            availableBlocks * blockSize
        } else {
            0
        }

    /**
     * 获取SDCARD总的存储空间
     *
     * @return
     */
    val totalExternalMemorySize: Long
        get() = if (externalMemoryAvailable()) {
            val path = Environment.getExternalStorageDirectory()
            val stat = StatFs(path.path)
            var blockSize: Long = 0
            blockSize =
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR2) {
                        stat.blockSizeLong
                    } else {
                        stat.blockSize.toLong()
                    }
            var totalBlocks: Long = 0
            totalBlocks =
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR2) {
                        stat.blockCountLong
                    } else {
                        stat.blockCount.toLong()
                    }
            totalBlocks * blockSize
        } else {
            0
        }

    /**
     * 获取手机内部剩余存储空间
     *
     * @return
     */
    val availableInternalMemorySize: Long
        get() {
            val path = Environment.getDataDirectory()
            val stat = StatFs(path.path)
            var blockSize: Long = 0
            blockSize = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR2) {
                stat.blockSizeLong
            } else {
                stat.blockSize.toLong()
            }
            var availableBlocks: Long = 0
            availableBlocks = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR2) {
                stat.availableBlocksLong
            } else {
                stat.availableBlocks.toLong()
            }
            return availableBlocks * blockSize
        }

    /**
     * 获取手机内部总的存储空间
     *
     * @return
     */
    val totalInternalMemorySize: Long
        get() {
            val path = Environment.getDataDirectory()
            val stat = StatFs(path.path)
            var blockSize: Long = 0
            blockSize =
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR2) {
                        stat.blockSizeLong
                    } else {
                        stat.blockSize.toLong()
                    }
            var totalBlocks: Long = 0
            totalBlocks = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR2) {
                stat.blockCountLong
            } else {
                stat.blockCount.toLong()
            }
            return totalBlocks * blockSize
        }
}